package org.nakedobjects.nos.store.hibernate.nof2hbm;

import java.util.Iterator;

import org.nakedobjects.nos.store.hibernate.testobjects.BiDirectional;
import org.nakedobjects.nos.store.hibernate.testobjects.ManyToMany;
import org.nakedobjects.nos.store.hibernate.testobjects.OneToMany;
import org.nakedobjects.nos.store.hibernate.testobjects.OneToOne;
import org.nakedobjects.nos.store.hibernate.testobjects.SimpleObject;
import org.nakedobjects.nos.store.hibernate.testobjects.SimpleSubClass;


public class PersistentNakedClassesTest extends SpecificationTestBase {

	public void testMapClasses() {
		system.addSpecification(BiDirectional.class.getName());
         final PersistentNakedClasses classes = PersistentNakedClasses.buildPersistentNakedClasses(Boolean.TRUE);
		
		int count = 0;
		for (Iterator iter = classes.getPersistentClasses(); iter.hasNext();) {
			iter.next();
			count++;
		}
		assertEquals("classes count",4,count);
		assertTrue(classes.isPersistentClass(BiDirectional.class.getName()));
		assertTrue(classes.isPersistentClass(ManyToMany.class.getName()));
		assertTrue(classes.isPersistentClass(OneToMany.class.getName()));
		assertTrue(classes.isPersistentClass(OneToOne.class.getName()));
		
		PersistentNakedClass bidirPc = classes.getPersistentClass(BiDirectional.class.getName());
		assertTrue("!assn secondOneToMany",!bidirPc.hasAssociation("secondonetomany"));
		assertTrue("assn oneToMany",bidirPc.hasAssociation("oneToMany"));
		assertTrue("assn manyToMany",bidirPc.hasAssociation("manyToMany"));
		assertTrue("assn oneToOne",bidirPc.hasAssociation("oneToOne"));

		Association assnManyToMany = bidirPc.getAssociation("manyToMany");
		assertTrue("!many to many inverse",!assnManyToMany.isInverse());
		assertEquals("many to many type", ManyToMany.class.getName(), assnManyToMany.getPersistentClass().getName());
		
		PersistentNakedClass m2mPc = classes.getPersistentClass(ManyToMany.class.getName());
		assertTrue("assn many",m2mPc.hasAssociation("many"));
		Association assnMany = m2mPc.getAssociation("many");
		assertTrue("many inverse",assnMany.isInverse());
		assertEquals("many type", BiDirectional.class.getName(), assnMany.getPersistentClass().getName());
		
	}
	public void testMapClassesNotBidirectional() {
		system.addSpecification(BiDirectional.class.getName());
         final PersistentNakedClasses classes = PersistentNakedClasses.buildPersistentNakedClasses(Boolean.FALSE);
		PersistentNakedClass pc = classes.getPersistentClass(BiDirectional.class.getName());
		assertTrue("!assn secondOneToMany",!pc.hasAssociation("secondonetomany"));
		assertTrue("assn oneToMany",pc.hasAssociation("oneToMany"));
		assertTrue("!assn manyToMany",!pc.hasAssociation("manytomany"));
		assertTrue("!assn oneToOne",!pc.hasAssociation("onetoone"));
	}
	
	public void testMapSubClasses() {
		system.addSpecification(SimpleSubClass.class.getName());
         final PersistentNakedClasses classes = PersistentNakedClasses.buildPersistentNakedClasses(Boolean.TRUE);
		
		int count = 0;
		for (Iterator iter = classes.getPersistentClasses(); iter.hasNext();) {
			iter.next();
			count++;
		}
		assertEquals("classes count",2,count);
		assertTrue(classes.isPersistentClass(SimpleSubClass.class.getName()));
		assertTrue(classes.isPersistentClass(SimpleObject.class.getName()));

		PersistentNakedClass pc = classes.getPersistentClass(SimpleObject.class.getName());
		PersistentNakedClass[] subclasses = pc.getSubClassesArray();
		assertEquals("subclasses size", 1, subclasses.length);
		assertEquals("subclass name", SimpleSubClass.class.getName(), subclasses[0].getName());
	}
}
// Copyright (c) Naked Objects Group Ltd.
